#!/usr/bin/env ruby

require 'rice/irc'
require 'rice/observer'

require 'nkf'

require 'util'
require 'mkdat'

NICK = 'Assam'
USER = ENV['USER'] || NICK
REAL = 'rice/assam'
CHANNEL = ['#kocha']
PASS = nil
#SERV = 'irc.friend.td.nu'
SERV = 'irc.himitsukichi.com'
PORT = 6667
VER = '0.1.1'
HAVE_UNAME = true

include RICE::Command

STDOUT.sync = true
STDERR.sync = true

print "IRC...\n"

o = RICE::SimpleClient.new(NICK, USER, REAL, PASS,
                           CHANNEL.collect {|i| NKF::nkf('-Ej', i)})
c = RICE::Connection.new(SERV, PORT)

c.add_observer(o)

def unsafe(msg)
  msg.gsub(/&quot;/, '"').gsub(/&gt;/, '>').gsub(/&lt;/, '<').gsub(/&amp;/, '&')
end

class IRCContext
  attr_reader :subject, :nick, :user, :time
  attr_accessor :target, :message

  def mine?
    nick == NICK && user =~ /#{USER}/
  end

  def initialize(subject, message)
    @subject = subject
    @target = message.params[0]
    @message = message.params[1..-1].join(' ')
    message.prefix =~ /^([^!]*)!(.*)/
    @nick = $1
    @user = $2
    if (@target == NICK)
      @target = @nick
    end
    @time = Time.now
  end

  def notice(msg)
    msg = NKF.nkf('-Ej', unsafe(msg))
    @subject.push(RICE::Command::notice(@target, msg))
  end
end

class Assam
  @@assam = nil

  attr_accessor :udb, :context

  def Assam.get
    unless @@assam
      @@assam = Assam.new
    end
    @@assam
  end
end

def response_for_say(nick, msg)
  msg = NKF.nkf('-Je', msg)
  append_chat_log(Assam.get.udb, NICK, safe("#{nick}: #{msg}"))
  mkdat(NICK)
end

def o.response_for_notice(subject, message)
  begin
    context = IRCContext.new(subject, message)
    return if context.mine?
    response_for_say(context.nick, context.message)
  rescue
    p $!.to_s
    print "#{$!.backtrace.join("\n")}\n"
  end
end

def o.response_for_privmsg(subject, message)
  begin
    context = IRCContext.new(subject, message)

    return if context.mine?

    msg = context.message

    if (msg[0] == 1 && msg[-1] == 1)
      ctcp = msg[1...-1]

      if ctcp =~ /^ACTION (.*)/
        msg = "#{context.nick} #{$1}"
      else
        if ctcp == 'VERSION' && context.target[0] != '#'
          print "CTCP VERSION from #{context.nick}\n"

          name = NICK
          ver = VER
          env = if (HAVE_UNAME)
                  `uname -s -r`.chomp.sub(/ /, '/')
                else
                  RUBY_PLATFORM
                end + " Ruby/" + RUBY_VERSION

          context.notice(1.chr + "VERSION #{name} #{ver} (#{env})" + 1.chr)

        elsif ctcp == 'CLIENTINFO' && context.target[0] != '#'
          print "CTCP CLIENTINFO from #{context.nick}\n"

          context.notice(1.chr + "CLIENTINFO :CLIENTINFO VERSION" + 1.chr)
        end

        return
      end
    end

    response_for_say(context.nick, msg)
  rescue
    p $!.to_s
    print "#{$!.backtrace.join("\n")}\n"
  end
end

def o.response_for_join(subject, message)
  begin
    context = IRCContext.new(subject, message)
    if context.mine?
      Assam.get.context = context
    end
  rescue
    p $!.to_s
    print "#{$!.backtrace.join("\n")}\n"
  end
end

trap(:USR1) do
  udb = get_udb
  udb.transaction do
    if !udb[NICK]
      login(udb, NICK)
    end
    if udb[NICK].key?(:stream)
      udb[NICK][:stream].each do |msg|
        msg.gsub!(/^.*?<\/em> /, '')
        msg.gsub!(/<[^>]*>/, '')
        if Assam.get.context
          Assam.get.context.notice(msg)
        end
      end
    end
    udb[NICK][:stream] = []
  end
end

print "Web...\n"

udb = get_udb
Assam.get.udb = udb
udb.transaction do
  login(udb, NICK)
  udb[NICK][:stream] = []
  udb[NICK][:pid] = $$
end

# def o.response_for_invite(subject, message)
#   nick = message.prefix
#   chan = message.params[1]

#   print "invited by #{nick} to #{chan}\n"

#   subject.push(join(chan))
# end

print "OK, booting.\n"

Thread.start do
  while true
    @udb.transaction do
      login(@udb, NICK)
    end
    sleep(270)
  end
end

begin
  c.start
rescue RICE::Connection::Closed
  p $!.to_s
  print "#{$!.backtrace.join("\n")}\n"
end


