require 'pstore'

require 'catalog'

def unescape(string)
  string.tr('+', ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
    [$1.delete('%')].pack('H*')
  end
end

def query
  r = {}
  ENV['QUERY_STRING'].split('&').each do |kv|
    kv =~ /^([^=]+)=(.*)/
    k = $1
    v = $2
    r[k] = unescape(v)
    r[k+"_"] = v
  end
  r
end

def safe(msg)
  return nil if !msg
  msg.gsub(/&/, '&amp;').gsub(/</, '&lt;').gsub(/>/, '&gt;').gsub(/"/, '&quot;')
end

def url(msg)
  msg.gsub(/((ftp|https|gopher|http|telnet):\/\/[^\s\n]+)/,
           '<a href="\1">\1</a>')
end

def get_udb
  udb = PStore.new('users.db')
  if !File.exist?('users.db')
    udb.transaction do
    end
  end
  udb
end

def get_users(udb)
  u = udb.roots.grep(/^[^_][^_]/).sort.join(', ')
  c(MEMBERS, u)
end

def get_topic(udb)
  c(TOPIC, udb['__topic'])
end

def get_hall
  return '' if !File.exists?('cache.txt')
  ret = ''
  File.open('cache.txt', 'r') do |i|
    i.flock(File::LOCK_SH)
    ret = i.read
    i.flock(File::LOCK_UN)
  end
  ret
end

def login(udb, u)
  addr = ENV['REMOTE_ADDR']
  if udb.root?(u)
    if udb[u][:addr] != addr
      raise "#{u}ϴ˻ѤƤ̾Ǥ"
    end
    udb[u][:last] = Time.now
  else
    udb[u] = {}
    udb[u][:addr] = addr
    udb[u][:last] = Time.now
    append_sys_log_nolock(udb, c(LOGIN_MSG, u))

    require 'plugin'
    plugin = PluginMgr.new(udb, u)
    plugin.apply_sys_plugin(:login)
  end
end

def sweep_timeout(udb)
  update = false
  now = Time.now
  udb.roots.each do |u|
    next if u =~ /^__/
    t = now - udb[u][:last]
    if !udb[u].key?(:pid) && ((udb[u].key?(:ajax) && t > 60) || t > 300)
      append_sys_log_nolock(udb, c(TIMEOUT_MSG, u))
      udb.delete(u)
      update = true

      require 'plugin'
      plugin = PluginMgr.new(udb, u)
      plugin.apply_sys_plugin(:timeout)
    end
  end
  update
end

def append_log(udb, msg)
  day = Time.now.day
  if !udb.root?('__logdump')
    udb['__logdump'] = day
  elsif udb['__logdump'] != day
    udb['__logdump'] = day

    require 'fileutils'
    require 'catalog'

    dat = File.open('chat.txt', 'r') do |ifile|
      ifile.readlines
    end.select {|l| l !~ /^<!-- OLD -->/}

    File.open('chat.txt', 'w') do |ofile|
      dat.reverse[0...20].reverse.each do |l|
        ofile.print "<!-- OLD -->#{l}"
      end
    end

    now = Time.now - 60*60*24
    year = now.year
    mon = now.mon
    day = now.day
    logdir = now.strftime("logs/%Y%m")
    FileUtils.mkdir_p(logdir)
    logfile = now.strftime"#{logdir}/%d.html"
    File.open(logfile, 'w') do |logf|
      logf.puts %Q(#{HEAD.sub(/kocha.css/, '../../kocha.css')}
</head>
<body>
<h1>#{TITLE} - #{now.strftime('%Y/%m/%d')}</h1>
<div class="log"><ul class="logList">
#{dat}</ul></div>
#{FOOT.gsub(/href="/, 'href="../../')}
)
    end
  end

  File.open('chat.txt', 'a') do |ofile|
    ofile.flock(File::LOCK_EX)
    ofile.puts(msg)
    ofile.flock(File::LOCK_UN)
  end
end

def append_stream(udb, user, msg)
  udb.roots.each do |u|
    next if u == user || u =~ /^__/
    if udb[u].key?(:stream)
      udb[u][:stream].push(msg)
    end
  end
end

def append_sys_log(udb, msg)
  udb.transaction do
    append_sys_log_nolock(udb, msg)
  end
end

def append_sys_log_nolock(udb, msg)
  msg = %Q(<li><em>#{Time.now.strftime('%H:%M:%S')}</em> <strong class="sys">#{msg}</strong>)
  append_log(udb, msg)
  append_stream(udb, nil, msg)
end

def append_chat_log(udb, user, msg)
  udb.transaction do
    append_chat_log_nolock(udb, user, msg)
  end
end

def append_chat_log_nolock(udb, user, msg)
  msg = %Q(<li><em>#{Time.now.strftime('%H:%M:%S')}</em> <strong>#{user}</strong>: #{msg})
  append_log(udb, msg)
  append_stream(udb, user, msg)
end

